;******************************************************************************
;	Subroutines USB-DEV					<pm_100.asm>
;
;	Tab_Desc:	USB-Descriptors
;	WR_9602:	Write a Data Byte into USBN9602 Register
;	RD_CUSB: 	get a Byte from USBN9602
;	SETUPPRO:	Setup Packet processing
;
;******************************************************************************

;==============================================================================
;	USB Descriptors definition
;==============================================================================
Tab_Desc
	addwf	PCL,F
	
	retlw	.18		; Size of this Descriptor
	retlw	1		; Descriptor Type = "Device"
	retlw	1		; USB Specification Release (Low)
	retlw	1		; USB Specification Release (High)
	retlw	0xFF		; Device Class
	retlw	0xFF		; Device SubClass
	retlw	0xFF		; Device Protocol
	retlw	8		; Maximum packet size for Endpoint_0
	retlw	0x11		; Vendor ID (Low)
	retlw	0		; Vendor ID (High)
	retlw	0x22		; Product (Low)
	retlw	0		; Product (High)
	retlw	0		; Device (Low)
	retlw	1		; Device (High)
	retlw	0		; Index of Manufactur Descriptor /None
	retlw	0		; Index of Product Descriptor /None
	retlw	0		; Index of Serial Number Descriptor /None
	retlw	1		; Number of possible Configurations
;------------------------------------------------------------------------------
Cfg_Desc			; USB-Configuration Descriptor
	retlw	.9		; Size of this Descriptor
	retlw	2		; Descriptor Type = "Configuration"
	retlw	LCD		; Total length (Low)
	retlw	0		; Total length (High)
	retlw	1		; Number of interfases
	retlw	1		; Number of this Configuration
	retlw	0		; Index of String Deccriptor
	retlw	0x80		; Attributes (Bitmap=Bus_Powered)
	retlw	.50		; Maximum power consuption (=100ma)

ITF_Desc
	retlw	0x09		;length of this desc.
	retlw	0x04		;INTERFACE descriptor
	retlw	0x00		;interface number
	retlw	0x00		;alternate setting
	retlw	0x06		;num. of (non 0) endpoints
	retlw	0x00		;interface class
	retlw	0x00		;interface subclass
	retlw	0x00		;interface protocol
	retlw	0x00		;index of intf. string

Pipe_0
	retlw	0x07		;length of this desc.
	retlw	0x05		;ENDPOINT descriptor
	retlw	0x81		;address (IN)
	retlw	0x02		;attributes (BULK)
	retlw	0x20		;max packet size (32)
	retlw	0x00		;
	retlw	0xFF		;interval (ms)
Pipe_1
	retlw	0x07		;length of this desc.
	retlw	0x05		;ENDPOINT descriptor
	retlw	0x02		;address (OUT)
	retlw	0x02		;attributes (BULK)
	retlw	0x20		;max packet size (32)
	retlw	0x00		;
	retlw	0xFF		;interval (ms)
Pipe_2
	retlw	0x07		;length of this desc.
	retlw	0x05		;ENDPOINT descriptor
	retlw	0x83		;address (IN)
	retlw	0x02		;attributes (BULK)
	retlw	0x20		;max packet size (32)
	retlw	0x00		;
	retlw	0xFF		;interval (ms)
Pipe_3
	retlw	0x07		;length of this desc.
	retlw	0x05		;ENDPOINT descriptor
	retlw	0x04		;address (OUT)
	retlw	0x02		;attributes (BULK)
	retlw	0x20		;max packet size (32)
	retlw	0x00		;
	retlw	0xFF		;interval (ms)
Pipe_4
	retlw	0x07		;length of this desc.
	retlw	0x05		;ENDPOINT descriptor
	retlw	0x85		;address (IN)
	retlw	0x02		;attributes (BULK)
	retlw	0x40		;max packet size (64)
	retlw	0x00		;
	retlw	0xFF		;interval (ms)
Pipe_5
	retlw	0x07		;length of this desc.
	retlw	0x05		;ENDPOINT descriptor
	retlw	0x06		;address (OUT)
	retlw	0x02		;attributes (BULK)
	retlw	0x40		;max packet size (64)
	retlw	0x00		;
	retlw	0xFF		;interval (ms)

;******************************************************************************

;==============================================================================
; 	Setup Packet processing
;------------------------------------------------------------------------------
;       Input   :       M-buffer with Setup Packet
;       Output  :	...
;	Variable:	FSR, TMP1, TMP2, ...
;------------------------------------------------------------------------------
SETUPPRO
	movlw	BUF_M0		; ready Pointer
	movwf	FSR		;
	movf	INDF,W		; get Request Type
	movwf	TMP2		;
	incf	FSR,F		; get Request
	movf	INDF,W		; 
	movwf	TMP1		;

	btfsc	TMP2,7		; Request Type=0?
	goto	REQT1		;  no, Request_Type=1
;------------------------------------------------------------------------------
	movlw	0x05		; Request=5?
	xorwf	TMP1,W		;
	btfsc	STATUS,Z	;	Z=1 if Request=5
	goto	SET_ADDR	;  yes
;------------------------------------------------------------------------------
	movlw	0x09		; Request=9?
	xorwf	TMP1,W		;
	btfss	STATUS,Z	;	Z=1 if Request=9
	goto	UN_USED		;  no

	WR_USB	EPC1,0x11	; Endpoint_1 Enable (Address = "1")
	WR_USB	EPC2,0x12	; Endpoint_2 Enable (Address = "2")
	WR_USB	EPC3,0x13	; Endpoint_3 Enable (Address = "3")
	WR_USB	EPC4,0x14	; Endpoint_4 Enable (Address = "4")
	WR_USB	EPC5,0x15	; Endpoint_5 Enable (Address = "5")
	WR_USB	EPC6,0x16	; Endpoint_6 Enable (Address = "6")

	WR_USB	RXC1,9		; RX_FIFO1 Flush & Enable
	WR_USB	RXC2,9		; RX_FIFO2 Flush & Enable (don't use)
	WR_USB	RXC3,9		; RX_FIFO3 Flush & Enable

	WR_USB	TXC0,8		; TX_FIFO0 Flush
	WR_USB	RXC0,8		; RX_FIFO0 Flush
	WR_USB	TXC0,5		; send ZERO-Packet by Endpoint_0

	bcf	RGST,DESCR	; clear Descriptor flag
	LED	ON		; PLUG Done

	bcf	T1CON,TMR1ON	;STOP TMR1
	clrf	PIR1		;
	return
;------------------------------------------------------------------------------
SET_ADDR
	WR_USB	TXC0,8		;Flush Endpoint_0 Transmiter
	WR_USB	RXC0,8		;Flush Endpoint_0 Receiver

	WR_USB	EPC0,0x40	; set flag "Function Address"
	
	WR_ADDR	FAR 		;set "Function Address"
	movlw	BUF_M2		; get Address (Low Byte)
	movwf	FSR		;
	movf	INDF,W		; 
	movwf	TMP1		;
	bsf	TMP1,7		; set bit Adress_Enable
	WR_REG	TMP1		; load Function Address into CUSB

	WR_USB	TXC0, 5		; send ZERO-Packet
	return
;------------------------------------------------------------------------------
REQT1
	movlw	6		;Request=6?
	xorwf	TMP1,W		;
	btfss	STATUS,Z	;	Z=1 if Request=6
	goto	UN_USED		; no

GET_DESC
	movlw	BUF_M0+3	; get & save Descriptor Type
	movwf	FSR		;
	movf	INDF,W		;
	movwf	TMP1		;

	movlw	1		; Descriptor Type="DEVICE"?
	xorwf	TMP1,W		;
	btfss	STATUS,Z	;	Z=1 if Request==
	goto	GET_D2		;  no, next test
	bsf	RGST,DESCR	;  yes,  set Descriptor flag
	bsf	TXST,TX0_TGL	;

	clrf	DESC_IDX	; Pointer initialize
	movlw	.18		; ready Answer's Length
	movwf	DESC_END	;
	call	TR_DESCR	; load Descriptor into FIFO0
	return

GET_D2
	movlw	2		; Descriptor Type="CONFIG"?
	xorwf	TMP1,W		;
	btfss	STATUS,Z	;	Z=1 if Request==
	goto	GET_D3		;  no, next test
	bsf	RGST,DESCR	;  yes, set Descriptor flag
	bsf	TXST,TX0_TGL	;

	movlw	.18		; Pointer initialize
	movwf	DESC_IDX	;

	movlw	LCD		; ready Maximum Answer's Length
	movwf	DESC_END	;

	movlw	BUF_M0+7	; Address of High Byte Descriptor's Length
	movwf	FSR		;
	movf	INDF,W		;High Byte Descriptor's Length=0?
	btfss	STATUS,Z	;	(Z=1 if result was 0)
	goto	GET_D2f		; no, Ld >=LCD

	movlw	BUF_M0+6	; Address of Low Byte Descriptor's Length
	movwf	FSR		;

	movlw	LCD		; ready Maximum Answer's Length
	movwf	DESC_END	;

	subwf	INDF,W		;Descriptor's Length <LCD? (W:=D_Length - LCD)
	btfsc	STATUS,C	;	(C=0 if result was negative)
	goto	GET_D2f		; no Ld >=LCD 

	movf	INDF,W		; load Descriptor's Length for Answer
	movwf	DESC_END	;
GET_D2f
	call	TR_DESCR	;	
	return			;

GET_D3
				;String or Interface or Endpoint? (don't use)
;------------------------------------------------------------------------------
UN_USED
	BUS_OUT			; set PORTB for output
	WR_ADDR	EPC0 		; Endpoint_0 Control Register
	BUS_IN			; set PORTB for input
	RD_DATA	TMP1		; load TMP1 from Endpoint_0 Control Register
	bsf	TMP1,7		; ready flag "STALL"
	BUS_OUT			; set PORTB for output
	WR_REG	TMP1		; set flag "STALL"
	return
;******************************************************************************

