;******************************************************************************
; Subroutines USB-MOD						<pm_cmd.asm>
;
;	COMMAND:	Command Processing
;******************************************************************************

;==============================================================================
;	Command Processing	(called from sub: USB_REC)
;------------------------------------------------------------------------------
;       Input   :       RX_FIFO1 with Command; TMP2=RXS1
;==============================================================================
COMMAND
	GET_RG	RXD1,TMP1	;load Command from RX_FIFO1

	btfsc	TMP1,7		;Command from Group "8x" ?
	goto    CMD_8X		; yes

	btfsc	TMP1,6		;Command from Group "4x" ?
	goto    CMD_4X		; yes

	btfsc	TMP1,5		;Command from Group "2x" ?
	goto    CMD_2X		; yes

	btfsc	TMP1,4		;Command from Group "1x" ?
	goto    CMD_1X		; yes

CMD_0X
	btfsc	TMP1,3		;Command="08" ?
	goto    CMD_08		; yes, NOP

	btfsc	TMP1,2		;Command="04" ?
	goto    CMD_04		; yes, NOP

	btfsc	TMP1,1		;Command="02" ?
	goto    CMD_02		; yes, NOP

	btfsc	TMP1,0		;Command="01" ?
	goto    CMD_01		; yes, NOP
;------------------------------------------------------------------------------
CMD_00				;NOP
CMD_01				;NOP
CMD_02				;NOP
CMD_04				;NOP
CMD_08				;NOP
	movlw	0x0F		;ready Error Code: "No Operation"
	movwf	ERRCODE		; 
	goto	SEND_ER		;Send Error Message to PC & Return
;==============================================================================
CMD_1X
	btfsc	TMP1,3		;Command="18" ?
	goto    CMD_18		; yes, NOP

	btfsc	TMP1,2		;Command="14" ?
	goto    CMD_14		; yes, NOP

	btfsc	TMP1,1		;Command="12" ?
	goto    CMD_12		; yes, NOP

	btfsc	TMP1,0		;Command="11" ?
	goto    CMD_11		; yes, NOP

CMD_10				;NOP
CMD_11				;NOP
CMD_12				;NOP
CMD_14				;NOP
CMD_18				;NOP
	movlw	0x0F		;ready Error Code: "No Operation"
	movwf	ERRCODE		; 
	goto	SEND_ER		;Send Error Message to PC & Return

;==============================================================================
CMD_2X
	btfsc	TMP1,3		;Command="28" ?
	goto    CMD_28		; yes, get 4 Bytes from PIC's Memo

	btfsc	TMP1,2		;Command="24" ?
	goto    CMD_24		; yes, ready & send Message "Status"

	btfsc	TMP1,1		;Command="22" ?
	goto    CMD_22		; yes, ready & send Message "Type & Version"

	btfsc	TMP1,0		;Command="21" ?
	goto    CMD_21		; yes, NOP

CMD_20				;NOP
CMD_21				;NOP
	movlw	0x0F		;ready Error Code: "No Operation"
	movwf	ERRCODE		; 
	goto	SEND_ER		;Send Error Message to PC & Return

;------------------------------------------------------------------------------
CMD_22				;ready & send Message "Type & Version"
	movlw	BUF_M0		; ready Pointer
	movwf	FSR		;

	movlw	.6		;Lp (M-buffer's Length =32)
	movwf	INDF		;
	incf	FSR,F		;
	movlw	0xF1		;Type of the Packet
	movwf	INDF		;
	incf	FSR,F		;
	movlw	C_TYPE		;Type of the Controller
	movwf	INDF		;
	incf	FSR,F		;
	movlw	C_VER		;Version of the Controller
	movwf	INDF		;
	incf	FSR,F		;
	movlw	DATE		;Date
	movwf	INDF		;
	incf	FSR,F		;
	movlw	MONTH		;Month
	movwf	INDF		;
	incf	FSR,F		;
	movlw	YEAR		;Year
	movwf	INDF		;

	call	TR_FIFO1	;load Message into FIFO1 & Transmit Enable
	return
;------------------------------------------------------------------------------
CMD_24				;ready & send Message "BST & My Status"

	movlw	BUF_M0		;ready Pointer
	movwf	FSR		;

	movlw	.3		;Lp (M-buffer's Length =32)
	movwf	INDF		;

	incf	FSR,F		;Type of Packet
	movlw	0xF2		;
	movwf	INDF		;

	incf	FSR,F		;BST
	movlw	0xF0		; Switches on <4:0>
	iorwf	PORTC,W		; ready Bits for LEDs
	movwf	INDF		;
	swapf	INDF,F		;
	comf	INDF,F		;

	incf	FSR,F		;Main Status register
	movf	RGST,W		; 
	movwf	INDF		;

	call	TR_FIFO1	;load Message into FIFO1 & Transmit Enable
	return
;------------------------------------------------------------------------------
CMD_28				;getMemo

	RD_DATA	FSR		;load Pointer from CUSB

        bsf     STATUS,RP0      ;select bank 1

	movlw	.6		;Lp (M-buffer's Length =32)
	movwf	BUF_M0-0x80	;
	movlw	0xF3		;Packet's Type
	movwf	BUF_M0-0x80+1	;
	movf	FSR,W		;Address
	movwf	BUF_M0-0x80+2	;
	movf	INDF,W		;Data 1
	movwf	BUF_M0-0x80+3	;
	incf	FSR,F		;Data 2
	movf	INDF,W		;
	movwf	BUF_M0-0x80+4	;
	incf	FSR,F		;Data 3
	movf	INDF,W		;
	movwf	BUF_M0-0x80+5	;
	incf	FSR,F		;Data 4
	movf	INDF,W		;
	movwf	BUF_M0-0x80+6	;

        bcf     STATUS,RP0      ;select bank 0

	call	TR_FIFO1	;load Message to FIFO1 & Transmit Enable
	return
;==============================================================================
CMD_4X
	btfsc	TMP1,3		;Command="48" ?
	goto    CMD_48		; yes, NOP

	btfsc	TMP1,2		;Command="44" ?
	goto    CMD_44		; yes, NOP

	btfsc	TMP1,1		;Command="42" ?
	goto    CMD_42		; yes, NOP

	btfsc	TMP1,0		;Command="41" ?
	goto    CMD_41		; yes, NOP

CMD_40				;NOP
CMD_41				;NOP
CMD_42				;NOP
CMD_44				;NOP
CMD_48				;NOP
	movlw	0x0F		;ready Error Code: "No Operation"
	movwf	ERRCODE		; 
	goto	SEND_ER		;Send Error Message to PC & Return
;==============================================================================
CMD_8X
	btfsc	TMP1,3		;Command="88" ?
	goto    CMD_88		; yes, NOP

	btfsc	TMP1,2		;Command="84" ?
	goto    CMD_84		; yes, NOP

	btfsc	TMP1,1		;Command="82" ?
	goto    CMD_82		; yes, NOP

	btfsc	TMP1,0		;Command="81" ?
	goto    CMD_81		; yes, NOP

CMD_80				;NOP
CMD_81				;NOP
CMD_82				;NOP
CMD_84				;NOP
CMD_88				;NOP
	movlw	0x0F		;ready Error Code: "No Operation"
	movwf	ERRCODE		; 
	goto	SEND_ER		;Send Error Message to PC & Return
;==============================================================================
SEND_ER
	movlw	BUF_M0		;ready Pointer
	movwf	FSR		;

	movlw	2		;ready Lp
	movwf	INDF		;
	incf	FSR,F		;
	movlw	0xE0		;ready Packet's Type
	movwf	INDF		;
	incf	FSR,F		;
	movf	ERRCODE,W	;ready Error Code
	movwf	INDF		;

	call	TR_FIFO1	;load Packet from M-buffer into TX_FIFO1
	return
;******************************************************************************

