;******************************************************************************
;								<pm_ini.asm>
;	PIC & CUSB Registers Initialize
;******************************************************************************

;=============================bank0============================================
        bcf     STATUS,RP0      ;select bank 0

	movlw	B'11110110'	;ready PORTA bits
	movwf	PORTA           ; PORTA<0>=LED0=ALE:=0, all other:=1
	
        movlw	0xFF		;ready PORTB bits
	movwf	PORTB           ; PORTB<7:0>:=1

	movlw	B'00000000'	;ready PORTC bits
	movwf	PORTC           ;  PORTC<7:1>:=0 (LIDs OFF)

;=============================bank1============================================
        bsf     STATUS,RP0      ;select bank 1

        movlw   B'11110000'	;PORTA<3:0>=LED0,RD,WR,ALE:=outputs
        movwf   TRISA-0x80	;

        movlw   0xFF		;PORTB<7:0>:=inputs
        movwf   TRISB-0x80	;

        movlw   B'00001111'	;PORTC<7:4>:=outputs, PORTC<3:0>:=inputs
        movwf   TRISC-0x80	;

        MOVLW   B'11101101'	;OPTION<2..0>=101: Kd(WDT)=32
;		  .   I-PSA=1: Prescaler assigned to the WDT, Kd(TMR0)=1
;		  .  I-TOSE=0: TMR0 Increment on low-to-high on pin_RA4
;		  . I-TOCS=1: TMR0 connect with pin_RA4
;		  .I-INTEDG=1: Interrupt on rising edge pin_RB0, if enabled
;		  I-PORTB pull-up disabled
        MOVWF   OPTION_REG-0x80	;

        movlw   B'00000000'     ;Interruptions:
;                 .      I-Interrupt flag RB<7:4>: RBIF
;                 .     I-Interrupt flag RB<0>: INTF
;                 .    I-Interrupt flag TMR0: TOIF
;                 .   I-control bit RBIE (0-. RBIF disabled)
;                 .  I-control bit INTE (0-. INTF disabled)
;                 . I-control bit T0IE (0-. TMR0 disabled)
;                 .I-control bit PEIE (0-All Peripherial Interrupts disabled)
;                 I-control bit GIE (0- All Interrupts disabled)
        movwf   INTCON

        movlw   B'00000000'     ;Control bits for the Peripherial Interrupts:
;                 .      I-TMR1IE =0 -Interrupt from TMR1 disabled
;                 .     I-TMR2IE =0 -Interrupt from TMR2 disabled
;                 .    I-CCP1IE =0 -Interrupt from Comparator1 disabled
;                 .   I-SSPIE =0 -Interrupt from SPI disabled
;                 .  I-TXIE =0 -Interrupt from TXIF (RS transmiter) disabled
;                 . I-RCIE =0 -Interrupt from RCIF (RS receiver) disabled
;                 .I-unused
;                 I-PSPIE =0 -Interrupt from Parallel slave port disabled
        movwf   PIE1-0x80	;

        clrf	PIE2-0x80	;Interrupt from Comparator2 disabled

	movlw	.250		;ready TMR2 period Register
	movwf	PR2-0x80	;

        movlw   B'11000001'     ;Transmit STATUS & CONTROL Register SCI (RS232)
;                 .      I-TXD8-9-th bit of transmit data, if enabled
;                 .     I-TRMT=1: Transmit shift Register empty
;                 .    I-BRGH=0: Baud rate is Low
;                 .   I-unused
;                 .  I-SYNC=0: Asynchronous mode
;                 . I-TXEN=0:				Transmit disabled
;                 .I-TX8/9=1: Transmit Data Length =9 bits
;                 I-CSRC=1: Master mode (Clock from BRGH)
        movwf   TXSTA-0x80	;

				;(Fosc=16.0 MHz)
;	movlw   .8		;Baud rate:	115.2 KBaud (BRGH=1)
;	movlw   .12		;		 19.2 KBaud (BRGH=0)
	movlw   .25	    	;		  9.6 KBaud (BRGH=0)
        movwf   SPBRG-0x80	;

;=============================bank0============================================
        bcf     STATUS,RP0      ; select bank 0
        
        movlw   B'01000000'     ; Receive STATUS & CONTROL Register SCI (RS232)
;                 .      I-RCD8: 9-th bit of Receive data, if enabled
;                 .     I-OERR: Overrun Error bit
;                 .    I-FERR: Framing Error bit
;                 .   I-unused
;                 .  I-CREN=0: Receive disabled
;                 . I-SREN: no used in Asynchronous mode
;                 .I-RC8/9: Receive Data Length =9 bits
;                 I-SPEN=0:		Serial port SCI (RS232) disabled
        movwf   RCSTA

	movlw	B'00010000'	;Synchronous Serial Port (SPI) Control Register
;                 .   IIII-Master-mode, clock=OSC/4
;             CKP .  I- High level (CLK =1) in Idle state
;                 . I-SSPEN=0: Synchronous Serial Port disabled
;           SSPOV .I-Receive overflow flag
;            WCOL I-Write collision detect
        movwf   SSPCON

        MOVLW   B'00100000'     ; TMR1 Control bits
;                 .      I-TMR1ON=0: TMR1 is OFF
;                 .     I-TMR1CS=0: Internal clock: =Fosc/4
;                 .    I-T1SYNC: no use if TMR1CS=0
;                 .   I-T1OSCEN=0: RC_Oscillator shut off
;                 . II-T1CKPS<1,0> =10: Prescale = 4 (Ti=1mks if Fosc=16MHz)
;                 II-unused
        MOVWF   T1CON

        MOVLW   B'00000010'     ; TMR2 Control bits
;                 .     II-T2CKPS<1,0>=10: Prescale = 16
;                 .    I-TMR2ON=0: TMR2 is OFF
;                 .IIII-TOUTPS<3..0>=0000: Postscale =1 (T=1 ms, if PR2=250)
;                 I-unused
        MOVWF   T2CON
;-------------------------------------------------------------------------------

				;Clear My Registers
	clrf	ERRCODE		; Error code
	clrf	RGST		; Main Status Register
	clrf	TXST		; Status of TX_FIFO
	clrf	FCOUNTER	; USB Frame Counter
	clrf	DESC_END	; Counter for Descriptor Transmitting
	clrf	DESC_IDX	; Descriptor Index (current offset)

;CUSB Registers Initialize
	WR_USB	MCNTRL,	1	;Main Control Register: Soft Reset

	WR_USB	CCONF,	2	;Set FCLK=16MHz //FCLK=48MHz/(k+1)

	call	START_LIGHT	;Two blinks of LED: _/200ms\100/200ms\_ 

STOP_DIS
	WR_USB	MCNTRL,	4	;"Voltage Regulator Enable"

	movlw	1		;1ms Delay
	call	WDELAY_TMR2	; Td = 1ms * [W] (F_SCLK=16M)

	WR_USB	FAR,	0x80	;Address 00 & flag "Address Enable"
	WR_USB	EPC0,	0	;clear flag "Default Address"

		; NAK Mask Register. Only OUT_0 enable
	WR_USB	NAKMSK,	B'00001000'

		; Transmit Mask Register. Enable: TXFIFO_3...0
	WR_USB	TXMSK,	B'00001111'

		; Receive Mask Register. Enable: RXFIFO_3:0
	WR_USB	RXMSK,	B'00001111'

		; Alternate Mask Register. Enable: Reset, SD5
	WR_USB	ALTMSK,	B'01100000'

		; Main Mask Register. Enable: RX_EV, NAK, TX_EV, ALT
	WR_USB	MAMSK,	B'01010110'

	WR_USB	TXC0,	8	;Endpoint_0 Flush & Transmit Disable
	WR_USB	RXC0,	1	;Endpoint_0 Receive Enable

	WR_USB	NFSR,	2	;Node Operational

		;Main Control Register: Node Attached & Voltage Regulator Enable
	WR_USB	MCNTRL,	B'00001100'

;*******************************************************************************

