;******************************************************************************
;	USB-MOD Subroutines					<pm_usbrd.asm>
;	
;	USB_REC:	Receive Event Processing
;	USBRESET:	USB Software Reset Event processing
;	USB_NAK:	NAK Event processing
;	USB_FRA:	FRAME Event processing
;******************************************************************************

;==============================================================================
;	Receive Event Processing
;==============================================================================
;       Input:       	None
;       Output:		
;			
;	Variable:	RV_EVENT, TMP2, TMP1
;------------------------------------------------------------------------------
USB_REC
	GET_RG	RXEV, EVENT	;load EVENT from Receive Event Register

	btfss	EVENT,3		;RX_FIFO3 Event?
	goto	USB_REC0	; no, next test
USB_REC3			; yes
	GET_RG	RXS3,TMP2	; load TMP2 from RX_FIFO3 Status Register

	btfsc	TMP2,7		;is it RX_Error?
	goto	REC3_F		; yes

	btfss	TMP2,4		;is it Packet completed?
	goto	REC3_F		; no

	movlw	0x0F		;ReceiveCOUNTer=0?
	andwf	TMP2,W		;
	btfsc	STATUS,Z	;
	goto	REC3_F		; yes, it is an Error
REC3_OK
	GET_RG	RXD3,PORTC	;load Data from RX_FIFO3
REC3_F
	WR_USB	RXC3,9		;RX_FIFO3 Flush & Enable
;------------------------------------------------------------------------------
USB_REC0
	btfss	EVENT,0		;RX_FIFO0 Event?
	goto	USB_REC1	;  no, next test
	GET_RG	RXS0, TMP2	;  yes, load TMP2 from Receive Status Register

	btfss	TMP2,6		; is it SETUP Packet?
	goto	RX0_FLUSH	;  no
RD0_SET				;  yes, read Setup Packet
	btfss	TMP2,4		; is it Packet completed?
	goto	USB_REC1	;  no, next test

	movlw	0x0F		;load COUNTER<3:0>
	andwf	TMP2,W		;
	movwf	COUNTER		;
	bz	RX0_FLUSH	;branch if COUNTER=0

RD_FIFO0
	movlw	BUF_M0		; ready Pointer for writing
	movwf	FSR		;

	BUS_OUT			;
	WR_ADDR	RXD0 		;
	BUS_IN			;

RL_FIFO0			;load M-buffer from FIFO_0
	RD_DATA	INDF		;
	incf	FSR,F		;
	decfsz	COUNTER,F	;
	goto	RL_FIFO0	;

CLR_EP0
	WR_USB	RXC0,	8	;FLUSH RX0, make sure the RX is off
	WR_USB	TXC0,	8	;FLUSH TX0, make sure the TX is off

	call	SETUPPRO	; Setup Packet processing

	clrf	RG_EVENT	; Event processing done
	return			;

RX0_FLUSH
	WR_USB	RXC0,	8	; FLUSH RX0, make sure the RX is off
	WR_USB	TXC0,	8	; FLUSH TX0, make sure the TX is off
	return			;
;------------------------------------------------------------------------------
USB_REC1
	btfss	EVENT,1		;RX_FIFO1 Event?
	goto	USB_REC2	; no, next test
	GET_RG	RXS1,TMP2	; yes, load TMP2 from  RXS1

	btfss	TMP2,4		;is it Packet completed?
	goto	USB_REC2	; no, next test

	call	COMMAND		;Command Processing

	WR_USB	RXC1,9		;Receive_CR_1: Flush & Enable
	goto	USB_REC2	;next Event test
;------------------------------------------------------------------------------
USB_REC2
	btfss	EVENT,2		;RX_FIFO2 Event?
	return			;  no, exit
	GET_RG	RXS2, TMP2	;  yes, load TMP2 from  RXS2 (clear RXS2)
	WR_USB	RXC2,9		;	RX_FIFO2 Flush & Enable
	return
;******************************************************************************

;==============================================================================
; 	USB Software Reset Event processing
;------------------------------------------------------------------------------
USBRESET
	BUS_OUT			;set PORTB for output
	WR_ADDR	NFSR		;set Node Reset State
	WR_CONS	0		;

	LED	OFF		;UnPLUG Done

	clrf	RGST		;UnPLUG & ECHO-mode
	clrf	TXST		;Status of TX_FIFO
	clrf	FCOUNTER	;USB Frame Counter
	clrf	DESC_END	;Counter for Descriptor Transmitting
	clrf	DESC_IDX	;Descriptor Index (current offset)

	WR_ADDR	FAR		;set AD_EN + ADDRESS=0
	WR_CONS	0x80		;

	WR_ADDR	EPC0		;clear  Endpoin_0 Control register
	WR_CONS	0		;

	WR_ADDR	TXC0		;FLUSH FIFO_0 & Transmit disable
	WR_CONS	8		;

	WR_ADDR	RXC0		;FIFO_0 Receiv enable
	WR_CONS	1		;

	WR_ADDR	NFSR		;set Node Functional State Register
	WR_CONS	2		;
	return
;******************************************************************************

;==============================================================================
;	NAK Event processing
;		NAKMSK=B'00011000' (only OUT_0 & IN_3 enable)
;==============================================================================
;       Input   :       None
;       Output  :       
;	Variable:	TMP1
;------------------------------------------------------------------------------
USB_NAK
	GET_RG	NAKEV, TMP1	; load TMP1 from NAK Event Register

	btfss	TMP1,4		; OUT_0 Event?
	goto	USB_NAK3	;  no, next test
	btfss	RGST,DESCR	;  yes, Descriptor was?
	return			;	no, there's nothing to be done
	bcf	RGST,DESCR	;	yes, clear Descriptor flag
	WR_USB	TXC0, 8		;		FLUSH FIFO_0
	WR_USB	RXC0, 1		;		FIFO_0 Receive Enable
	return

USB_NAK3
	btfss	TMP1,3		;IN_3 Event?
	return			; no, there's nothing to be done
	return			; yes, there's nothing to be done

;******************************************************************************

USB_FRA				;FRAME Event processing
	BUS_OUT			;
	WR_ADDR	TXC0 		;
	BUS_IN			;
	RD_DATA	TMP1		; 

	btfss	TMP1,0		;TX0 Enable?
	return

	incf	FCOUNTER,F

	btfss	FCOUNTER,2
	return

	WR_USB	TXC0,	8	;FLUSHTX0, make sure the TX is off
	WR_USB	RXC0,	9	;FLUSHRX0, make sure the RX is off
	clrf	FCOUNTER
	return
;******************************************************************************

