;******************************************************************************
;	USB-MOD Subroutines					<pm_usbtr.asm>
;
;	USB_TREV:	Transmit Event processing
;
;	TR_DESCR:	Descriptor Transmiting (TX_FIFO0)
;	TR_FIFO1:	send Packet  from M-buffer by TX_FIFO1
;	SEND_ER:	send Error Message to PC by TR_FIFO1
;******************************************************************************

;==============================================================================
;	Transmit Event processing
;------------------------------------------------------------------------------
;	Input	:	MAEV,TX_EV=1 (RG_EVENT)
;	Output	:	
;
;			
;	Variable:	EVENT, TMP2
;==============================================================================
USB_TREV
	GET_RG	TXEV, EVENT	;load EVENT from Transmit Event Register

	btfss	EVENT,3		;TX_FIFO3 Event?
	goto	USB_TREV0	; no, next test

USB_TREV3
	GET_RG	TXS3, TMP2	;load TMP2 from TX_FIFO3 Status Register

	btfss	TMP2,5		;Transmit done (TX_DONE=1)?
	goto	USB_TREV0	; no, next test

	bcf	TXST,TR3_EN	;TX_FIFO3 is free for loading

	movlw	TX3mTGL		;change TOGGLE
	xorwf	TXST,F		;

;------------------------------------------------------------------------------
USB_TREV0
	btfss	EVENT,0		; Endpoint_0 Event?
	goto	USB_TREV1	;  no, next test

	GET_RG	TXS0, TMP2	; load TMP2 from EP_0 Transmit Status Register

	btfss	TMP2,5		; Transmit done (TX_DONE=1)?
	goto	USB_TREV1	;  no, next FIFO test
	WR_USB	TXC0, 8		;  yes,	Flush FIFO_0 & Transmit Disable

	btfss	TMP2,6		; Transmit OK (ACK_STAT=1)?
	goto	TREV0_M1	;  no

	btfss	RGST,DESCR	; Descriptor transmiting?
	goto	TREV0_M1	;  no

	call	TR_DESCR	; Descriptor Transmiting
	goto	USB_TREV1	;

TREV0_M1
	WR_USB	RXC0, 1		; FIFO_0 Receive Enable
;------------------------------------------------------------------------------
USB_TREV1
	btfss	EVENT,1		;TX_FIFO1 Event?
	goto	USB_TREV2	; no, next test

	bcf	TXST,TR1_EN	;TX_FIFO1 is free
	GET_RG	TXS1, TMP2	;read (clear) TX_FIFO1 Status Register
;------------------------------------------------------------------------------
USB_TREV2
	btfss	EVENT,2		;TX_FIFO2 Event? (non used)
	return			;  no, exit
	GET_RG	TXS2, TMP2	;  yes, read (clear) Transmit Status Register
	return
;******************************************************************************

;==============================================================================
;	Descriptor Transmiting by Endpoint_0
;==============================================================================
;       Input   :       DESC_FL, DESC_IDX, DESC_END
;       Output  :       
;	Variable:	TMP2, COUNTER, DESC_FL, DESC_IDX, DESC_END
;------------------------------------------------------------------------------
TR_DESCR
	clrf	FCOUNTER	;

	BUS_OUT			;
	WR_ADDR	TXC0 		; Endpoint_0 Transmit Command Register
	WR_CONS	8		; Flush FIFO_0 & Transmit Disable

	WR_ADDR	TXD0 		; Endpoint_0 Transmit Data Register

	movf	DESC_END,W	;DESC_END=0?
	btfss	STATUS,Z	;
	goto	$+3		; no
	bcf	RGST,DESCR	; yes, 0_Packet transmiting
	goto	TRD_ENBL	;
	
	sublw	7		;DESC_END=1...7? (W:=7-W)
	btfsc	STATUS,C	;
	goto	TRD_LAST	; yes (C=1)
	movlw	8		; no  (C=0)
	movwf	COUNTER		;
	subwf	DESC_END,F	;
	goto	TRD_LOOP	;

TRD_LAST
	movf	DESC_END,W	; (DESC_END=1...7)
	movwf	COUNTER		;
	clrf	DESC_END	;
	bcf	RGST,DESCR	; Transmit Last Packet

TRD_LOOP
	movf	DESC_IDX,W	;
	call	Tab_Desc	; W:= Byte from Table

	WR_W_USB		; Write W into RG CUSB

	incf	DESC_IDX,F	;
	decfsz	COUNTER,F	;
	goto	TRD_LOOP	;

TRD_ENBL
	WR_ADDR	TXC0		; Endpoint_0 Transmit Command Register

	movlw	1		;TOGGLE (if necessary) & Transmit Enable
	btfsc	TXST,TX0_TGL	;
	movlw	5		;
	WR_W_USB		;

	movlw	TX0mTGL		;change TOGGLE
	xorwf	TXST,F		;
	return
;******************************************************************************

;==============================================================================
;	load Packet from M-buffer into Endpoint_1 & Transmit Enable
;==============================================================================
;       Input   :       M-buffer with Packet for PC
;			 (BUF_M0=Packet's Length = 0...31)
;       Output  :       Packet into Endpoint_1 (with Packet's Length)
;			TXST,TR1_EN=1
;	Variable:	COUNTER, FSR, Toggle
;------------------------------------------------------------------------------
TR_FIFO1
	btfsc	TXST,TR1_EN	;is TX_FIFO1 busy?
	return			; yes

	BUS_OUT			;set PORTB for output
	WR_ADDR	TXC1 		;Flush Endpoint_1 Transmiter
	WR_CONS	8		;

	movlw	BUF_M0		;Packet's Length Address
	movwf	FSR		;Pointer

	movlw	0x1F		;Mask, just in case
	andwf	INDF,W		;get Packet's Length
	movwf	COUNTER		;ready Loop Counter

	incf	COUNTER,F	;Byte of Lp must be taken into account

	WR_ADDR	TXD1 		;Endpoint_1 Transmit Data Register Address

TR_F1_L
	WR_REG	INDF		;Load FIFO from M-buffer
	incf	FSR,F		;Pointer increment

	decfsz	COUNTER,F	;Loop
	goto	TR_F1_L		;

	WR_ADDR	TXC1 		;Endpoint_1 Transmit Command Register Address

TR_F1_EN
	movlw	3		;ready PID=Data0 + TX_Last + TX_EN

	btfsc	TXST,TX1_TGL	;TOGGLE=1?
	movlw	7		; yes, set PID=Data1 + TX_Last + TX_EN

	WR_W_USB		;Transmit Enable (Write W into CUSB)

	movlw	TX1mTGL		;change TOGGLE
	xorwf	TXST,F		;

	bsf	TXST,TR1_EN	;TX_FIFO1 is full
	return
;******************************************************************************

