;******************************************************************************
;	Subroutines USB-DEV					<pm_util.asm>
;
;	WR_9602:	Write a Data Byte into USBN9602 Register
;	RD_CUSB: 	get a Byte from USBN9602
;	WDELAY_TMR2:	Delay: Td = 1ms  * [W]
;	START_LIGHT:	Two blinks of the LED: _/150ms\_/150ms\_
;******************************************************************************

;==============================================================================
;	Write Data Byte to USBN9602 Register (is used by MACRO "WR_USB")
;==============================================================================
;       Input   :       W= USB Register Address, USB_DATA= Byte for loading
;       Output  :       USBN9602 Register with new Data
;	Variable:	TRISB=0
;------------------------------------------------------------------------------
WR_9602
				; set PORTB for output
	bsf     STATUS,RP0      ;  select bank 1
	clrf	0x06		;  (TRISB) PORTB<7:0>:=outputs
	bcf     STATUS,RP0      ;  select bank 0

	movwf	PORTB		; Write Address from W into CUSB
	bsf	PORTA,ALE	;
	nop			;
	bcf	PORTA,ALE	;

	movf	USB_DATA,W	; Write Data from USB_DATA into CUSB
	movwf	PORTB		;
	bcf	PORTA,nWR	;
	nop			;
	bsf	PORTA,nWR	;
	return
;******************************************************************************

;==============================================================================
;	get Byte from USBN9602 (is used by MACRO "GET_RG")
;==============================================================================
;       Input   :       W= USB Register Address
;       Output  :       W= Byte from USBN9602
;	Variable:	TRISB
;------------------------------------------------------------------------------
RD_CUSB
				;set PORTB for output
	bsf     STATUS,RP0      ; select bank 1
	clrf	0x06		; (TRISB) PORTB<7:0>:=outputs
	bcf     STATUS,RP0      ; select bank 0

	movwf	PORTB		;Write Address from W into CUSB
	bsf	PORTA,ALE	;
	nop			;
	bcf	PORTA,ALE	;

				;set PORTB for input
	bsf     STATUS,RP0      ; select bank 1
	movlw	0xFF		;
	movwf	0x06		; (TRISB) PORTB<7:0>:=inputs
	bcf     STATUS,RP0      ; select bank 0

	bcf	PORTA,nRD	;read Data from CUSB
	nop			;
	movf	PORTB,W		;
	bsf	PORTA,nRD	;
	return
;******************************************************************************

;==============================================================================
WDELAY_TMR2			; Td = 1ms * [W] <if F_SCLK=16MHz>
	movwf	DCOUNTER	; ready Counter
	clrf	TMR2		; ready TMR2
	bcf	PIR1,TMR2IF	;
        bsf	T2CON,TMR2ON	; TMR2 start
	btfss	PIR1,TMR2IF	; wating TMR2IF=1 (1ms if F_SCLK=16MHz)
	goto	$-1		;
        bcf	PIR1,TMR2IF	; clear TMR2 flag
	decfsz	DCOUNTER,F	; Loop
	goto	$-4		;
        bcf	T2CON,TMR2ON	; TMR2 stop
	return
;******************************************************************************

;==============================================================================
;	Two blinks of the LED: _/150ms\_/150ms\_ 
;==============================================================================
START_LIGHT
	LED	ON		;150 ms Light
	movlw	.150		;
	call	WDELAY_TMR2	; Td = 1ms  * [W]

	LED	OFF		;100 ms Dark
	movlw	.100		;Delay 100 ms
	call	WDELAY_TMR2	; Td = 1ms  * [W]

	LED	ON		;150 ms Light
	movlw	.150		;
	call	WDELAY_TMR2	; Td = 1ms  * [W]
	LED	OFF		;
	return
;******************************************************************************

