;******************************************************************************
;	Main Module 					       "usb_k.asm"
;
;	Communication between PC (USB) and the Controller (4 LEDs + 4 switches)
;
;******************************************************************************
	list P=16C63
	include	<p16c63.inc>	; PIC's Registers definition

        __FUSES _CP_OFF&_PWRTE_ON&_WDT_ON&_HS_OSC ; set Configuration:
;                 I          I       I       select HS oscillator
;                 I          I       Watchdog Timer enable
;                 I          Power-up Timer enable (T_RESET=72ms+1024*Tosc)
;                 Code protection OFF


;---------------------- Bank_0 RAM --------------------------------------------

;		EQU	0x20	;from 0x21 to 0x69 are free

RGST		EQU	0x6A	;Main Status Register
TXST		EQU     0x6B	;Status TX_FIFOs
;		EQU	0x6C	;

EVENT		EQU     0x6D	;USB Event TMP Storage Register
RG_EVENT	EQU     0x6E	;USB Event Storage Register
USB_DATA	EQU	0x6F	;Register for Communication with USBN9602

				;is used only in USB_PLUG mode 
DESC_END	EQU     0x70	; Counter for Descriptor Transmitting
DESC_IDX	EQU     0x71	; Descriptor Index (current offset)

ERRCODE		EQU	0x72	;Error code
FCOUNTER	EQU	0x73	;USB frames Counter
DCOUNTER        EQU     0x74    ;Counter for Delay
TMP2            EQU     0x75    ;Common Register
TMP1            EQU     0x76    ;Common Register
COUNTER         EQU     0x77    ;Common Loop Counter

;		EQU     0x78	;0x78...0x7F are free


;---------------------- Bank_1 RAM --------------------------------------------

				;Packet for PC (by TX_FIFO1)
BUF_M0		EQU	0xA0	; Packet's Length (0...32)
BUF_M1		EQU	0xA1	; Packet's Type
BUF_M2		EQU	0xA2	; 1-st Byte of Data
;			0xBF	; the last

;		EQU	0xC0	;from 0xC0 to 0xFF are free



;----------------------- RGST Bits --------------------------------------------
DESCR		EQU	0	; ..=1: Descriptor transmitting by TX_FIFO0


;----------------------- TXST Bits --------------------------------------------
TR0_EN		EQU	0	; ..=1: TX_FIFO0 is full & TR_Enable (non used)
TR1_EN		EQU	1	; ..=1: TX_FIFO1 is full & TR_Enable
TR2_EN		EQU	2	; ..=1: TX_FIFO2 is full & TR_Enable (non used)
TR3_EN		EQU	3	; ..=1: TX_FIFO3 is full & TR_Enable
TX0_TGL		EQU	4	; Toggle flag for TX_FIFO0
TX1_TGL		EQU	5	; Toggle flag for TX_FIFO1
TX2_TGL		EQU	6	; Toggle flag for TX_FIFO2
TX3_TGL		EQU	7	; Toggle flag for TX_FIFO3

	constant TX0mTGL =0x10	; Mask for TX_FIFO0 Toggle
	constant TX1mTGL =0x20	; Mask for TX_FIFO1 Toggle
	constant TX2mTGL =0x40	; Mask for TX_FIFO2 Toggle
	constant TX3mTGL =0x80	; Mask for TX_FIFO3 Toggle

;----------------------- PORTA Bits -------------------------------------------
ALE		EQU     0	;Address Latch Enable		/out
nWR		EQU     1	;Active-Low write signal	/out
nRD		EQU     2	;Active-Low read signal		/out
LED0		EQU     3	;Active-Hi control signal	/out

;----------------------- PORTC Bits -------------------------------------------


;======================= Common Constants =====================================
	constant ON	=1
	constant OFF	=0

;======================= CUSB Addresses =======================================
	constant MCNTRL	=0x00	; Main Control Register
	constant CCONF	=0x01	; Clock Configuration Register
	constant RID	=0x03	; Revision Identifier Register
	constant FAR	=0x04	; Function Address Register
	constant NFSR	=0x05	; Node Functional State Register
	constant MAEV	=0x06	; Main Event Register
	constant MAMSK	=0x07	; Main Mask Register
	constant ALTEV	=0x08	; Alternate Event Register
	constant ALTMSK	=0x09	; Alternate Mask Register
	constant TXEV	=0x0A	; Transmit Event Register
	constant TXMSK	=0x0B	; Transmit Mask Register
	constant RXEV	=0x0C	; Receive Event Register
	constant RXMSK	=0x0D	; Receive Mask Register
	constant NAKEV	=0x0E	; Negative Acknowledgement Event Register
	constant NAKMSK	=0x0F	; NAK Mask Register
	constant FWEV	=0x10	; FIFO Warning Event Register

	constant EPC0	=0x20	; Endpoint Control Register_0
	constant TXD0	=0x21	; Transmit Data Register_0
	constant TXS0	=0x22	; Transmit Status Register_0
	constant TXC0	=0x23	; Transmit Command Register_0
	constant RXD0	=0x25	; Receive Data Register_0
	constant RXS0	=0x26	; Receive Status Register_0
	constant RXC0	=0x27	; Receive Command Register_0

	constant EPC1	=0x28	; Endpoint Control Register_1
	constant TXD1	=0x29	; Transmit Data Register_1
	constant TXS1	=0x2A	; Transmit Status Register_1
	constant TXC1	=0x2B	; Transmit Command Register_1

	constant EPC2	=0x2C	; Endpoint Control Register_2
	constant RXD1	=0x2D	; Receive Data Register_1
	constant RXS1	=0x2E	; Receive Status Register_1
	constant RXC1	=0x2F	; Receive Command Register_1

	constant EPC3	=0x30	; Endpoint Control Register_3
	constant TXD2	=0x31	; Transmit Data Register_2
	constant TXS2	=0x32	; Transmit Status Register_2
	constant TXC2	=0x33	; Transmit Command Register_2

	constant EPC4	=0x34	; Endpoint Control Register_4
	constant RXD2	=0x35	; Receive Data Register_2
	constant RXS2	=0x36	; Receive Status Register_2
	constant RXC2	=0x37	; Receive Command Register_2

	constant EPC5	=0x38	; Endpoint Control Register_5
	constant TXD3	=0x39	; Transmit Data Register_3
	constant TXS3	=0x3A	; Transmit Status Register_3
	constant TXC3	=0x3B	; Transmit Command Register_3

	constant EPC6	=0x3C	; Endpoint Control Register_6
	constant RXD3	=0x3D	; Receive Data Register_3
	constant RXS3	=0x3E	; Receive Status Register_3
	constant RXC3	=0x3F	; Receive Command Register_3


;================== Controller's Parameters =================================

	constant LCD	=.60	;USB Config Descriptor Length
	
	constant C_TYPE	=0x01	;Type of the Controller
	constant C_VER	=0x00	;Version of the Controller
	constant DATE	=0x20	;Date of creation (editing)
	constant MONTH	=0x07	;Month
	constant YEAR	=0x99	;Year

;============================================================================
;============================================================================
;                           START SECTION
;============================================================================
        org	0
	goto	BEGIN
	
	org	0x08
	de	"USB_k_01"
	de	"20.07.99"
	de	"Pic16c63"

;==============================================================================
;			SUBROUTINES SECTION
;==============================================================================
	include <pm_macro.asm>	; definition of the MACROS

	include <pm_100.asm>	; Tab_Desc: USB Descriptors (18+60 Bytes)
				; WR_9602:  write Data Byte to USBN9602 Register
				; RD_CUSB:  get Byte from USBN9602
				; SETUPPRO: Setup Packet processing

	include <pm_cmd.asm>	; COMMAND: Command Processing (RX_FIFO1)

	include <pm_usbrd.asm>	; USB_REC:  Receive Event Processing
				; USBRESET: USB Software Reset Event Processing
				; USB_NAK:  NAK Event processing
				; USB_FRA:  FRAME Event processing
				; RD_FIFO0: Load M-buffer from FIFO_0

	include <pm_usbtr.asm>	; USB_TREV: Transmit Event processing
				; TR_DESCR: Descriptor Transmiting (TX_FIFO0)
				; TR_FIFO1: send Packet to PC from M-buffer


	include <pm_util.asm>	; WR_9602:  write Data Byte to USBN9602 Register
				; RD_CUSB:  get Byte from USBN9602
				; WDELAY_TMR2:	Td = 1ms  * [W]
				; START_LIGHT:	Two blinks of the LED (400ms)

;============================================================================
;                           WORK SECTION
;============================================================================
BEGIN
	include <pm_ini.asm>	;Initialize

;Serial port SCI (RS232) disabled
;Synchronous Serial Port disabled
;TMR0 unused
;TMR1 unused
;TMR2 is used for Delay (sub "WDELAY_TMR2" Td = 1 ms * [W]) 
;All Interrupts disabled

;------------------------------------------------------------------------------
M_LOOP
	clrwdt			;

Test_USB_Events
	BUS_OUT			;load RG_EVENT from MAEV
	WR_ADDR	MAEV		;
	BUS_IN			;
	RD_DATA	RG_EVENT	;

	movlw	B'01010110'	;RX_EV or NAK or TX_EV or ALT Event?
	andwf	RG_EVENT,W	;
	btfsc	STATUS,Z	;
	goto	M_LOOP		; no, None Event

	btfsc	RG_EVENT,6	;Receive Event? (RG_EVENT=[MAEV])
	call	USB_REC		; yes, Receive Event processing

	btfsc	RG_EVENT,4	;NAK Event? (RG_EVENT=[MAEV])
	call	USB_NAK		; yes,	NAK Event processing

	btfsc	RG_EVENT,3	;FRAME Event?
	call	USB_FRA		; yes,	FRAME Event processing

	btfsc	RG_EVENT,2	;Transmit Event? (RG_EVENT=[MAEV])
	call	USB_TREV	; yes, Transmit Event processing

	btfss	RG_EVENT,1	;Alternate Event? (RG_EVENT=[MAEV])
	goto	M_LOOP		; no
;------------------------------------------------------------------------------
ALTEV_AN			;Alternate Event processing
	BUS_OUT			; set PORTB for output
	WR_ADDR	ALTEV		; Write Address
	BUS_IN			; set PORTB for input
	RD_DATA	EVENT		; load EVENT from ALTEV

ALTEV_6
	btfss	EVENT,6		;Signal Reset?
	goto	ALTEV_5		; no, test next bit
	call	USBRESET	; yes, Reset
	goto	M_LOOP		;

ALTEV_5
	btfsc	EVENT,5		;5ms without USB-Communications?
	nop			; yes, call Your sub, if you want
	goto	M_LOOP		;
;******************************************************************************
        end
